@@ -35,6 +35,7 @@
 #endif
 
 #include <alloca.h>
+#include <dirname.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -42,6 +43,7 @@
 
 #include "libguile/_scm.h"
 #include "libguile/smob.h"
+#include "libguile/fdes-finalizers.h"
 #include "libguile/feature.h"
 #include "libguile/fports.h"
 #include "libguile/strings.h"
@@ -289,6 +291,7 @@ SCM_DEFINE (scm_close, "close", 1, 0, 0,
     return scm_close_port (fd_or_port);
   fd = scm_to_int (fd_or_port);
   scm_evict_ports (fd);		/* see scsh manual.  */
+  scm_run_fdes_finalizers (fd);
   SCM_SYSCALL (rv = close (fd));
   /* following scsh, closing an already closed file descriptor is
      not an error.  */
@@ -311,6 +314,7 @@ SCM_DEFINE (scm_close_fdes, "close-fdes", 1, 0, 0,
   int rv;
 
   c_fd = scm_to_int (fd);
+  scm_run_fdes_finalizers (c_fd);
   SCM_SYSCALL (rv = close (c_fd));
   if (rv < 0)
     SCM_SYSERROR;
@@ -1439,8 +1443,9 @@ SCM_DEFINE (scm_umask, "umask", 0, 1, 0,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_mkstemp, "mkstemp!", 1, 0, 0,
-	    (SCM tmpl),
+SCM_INTERNAL SCM scm_i_mkstemp (SCM, SCM);
+SCM_DEFINE (scm_i_mkstemp, "mkstemp!", 1, 1, 0,
+	    (SCM tmpl, SCM mode),
 	    "Create a new unique file in the file system and return a new\n"
 	    "buffered port open for reading and writing to the file.\n"
 	    "\n"
@@ -1459,18 +1464,38 @@ SCM_DEFINE (scm_mkstemp, "mkstemp!", 1, 0, 0,
 	    "(let ((port (mkstemp! (string-copy \"/tmp/myfile-XXXXXX\"))))\n"
 	    "  (chmod port (logand #o666 (lognot (umask))))\n"
 	    "  ...)\n"
-	    "@end example")
-#define FUNC_NAME s_scm_mkstemp
+	    "@end example\n"
+            "\n"
+            "The optional @var{mode} argument specifies a mode, as a string\n"
+            "in the same format that @code{open-file} takes.  It defaults\n"
+            "to @code{\"w+\"}.")
+#define FUNC_NAME s_scm_i_mkstemp
 {
   char *c_tmpl;
+  long mode_bits;
   int rv;
+  int open_flags, is_binary;
+  SCM port;
 
   scm_dynwind_begin (0);
 
   c_tmpl = scm_to_locale_string (tmpl);
   scm_dynwind_free (c_tmpl);
+  if (SCM_UNBNDP (mode))
+    {
+      /* mkostemp will create a read/write file and add on additional
+         flags; open_flags just adjoins flags to that set.  */
+      open_flags = 0;
+      is_binary = 0;
+      mode_bits = SCM_OPN | SCM_RDNG | SCM_WRTNG;
+    }
+  else
+    {
+      open_flags = scm_i_mode_to_open_flags (mode, &is_binary, FUNC_NAME);
+      mode_bits = scm_i_mode_bits (mode);
+    }
 
-  SCM_SYSCALL (rv = mkstemp (c_tmpl));
+  SCM_SYSCALL (rv = mkostemp (c_tmpl, open_flags));
   if (rv == -1)
     SCM_SYSERROR;
 
@@ -1479,10 +1504,22 @@ SCM_DEFINE (scm_mkstemp, "mkstemp!", 1, 0, 0,
 			tmpl, SCM_INUM0);
 
   scm_dynwind_end ();
-  return scm_fdes_to_port (rv, "w+", tmpl);
+
+  port = scm_i_fdes_to_port (rv, mode_bits, tmpl);
+  if (is_binary)
+    /* Use the binary-friendly ISO-8859-1 encoding. */
+    scm_i_set_port_encoding_x (port, NULL);
+
+  return port;
 }
 #undef FUNC_NAME
 
+SCM
+scm_mkstemp (SCM tmpl)
+{
+  return scm_i_mkstemp (tmpl, SCM_UNDEFINED);
+}
+
 
 /* Filename manipulation */
 
@@ -1513,31 +1550,22 @@ SCM_DEFINE (scm_dirname, "dirname", 1, 0, 0,
 	    "component, @code{.} is returned.")
 #define FUNC_NAME s_scm_dirname
 {
-  long int i;
-  unsigned long int len;
+  char *c_filename, *c_dirname;
+  SCM res;
 
-  SCM_VALIDATE_STRING (1, filename);
+  scm_dynwind_begin (0);
+  c_filename = scm_to_utf8_string (filename);
+  scm_dynwind_free (c_filename);
 
-  len = scm_i_string_length (filename);
+  c_dirname = mdir_name (c_filename);
+  if (!c_dirname)
+    SCM_SYSERROR;
+  scm_dynwind_free (c_dirname);
 
-  i = len - 1;
+  res = scm_from_utf8_string (c_dirname);
+  scm_dynwind_end ();
 
-  while (i >= 0 && is_file_name_separator (scm_c_string_ref (filename, i)))
-    --i;
-  while (i >= 0 && !is_file_name_separator (scm_c_string_ref (filename, i)))
-    --i;
-  while (i >= 0 && is_file_name_separator (scm_c_string_ref (filename, i)))
-    --i;
-
-  if (i < 0)
-    {
-      if (len > 0 && is_file_name_separator (scm_c_string_ref (filename, 0)))
-	return scm_c_substring (filename, 0, 1);
-      else
-	return scm_dot_string;
-    }
-  else
-    return scm_c_substring (filename, 0, i + 1);
+  return res;
 }
 #undef FUNC_NAME
 
@@ -1549,42 +1577,28 @@ SCM_DEFINE (scm_basename, "basename", 1, 1, 0,
 	    "@var{filename}, it is removed also.")
 #define FUNC_NAME s_scm_basename
 {
-  int i, j, len, end;
+  char *c_filename, *c_last_component;
+  SCM res;
 
-  SCM_VALIDATE_STRING (1, filename);
-  len = scm_i_string_length (filename);
+  scm_dynwind_begin (0);
+  c_filename = scm_to_utf8_string (filename);
+  scm_dynwind_free (c_filename);
 
-  if (SCM_UNBNDP (suffix))
-    j = -1;
+  c_last_component = last_component (c_filename);
+  if (!c_last_component)
+    res = filename;
   else
-    {
-      SCM_VALIDATE_STRING (2, suffix);
-      j = scm_i_string_length (suffix) - 1;
-    }
-  i = len - 1;
-  while (i >= 0 && is_file_name_separator (scm_c_string_ref (filename, i)))
-    --i;
-  end = i;
-  while (i >= 0 && j >= 0 
-	 && (scm_i_string_ref (filename, i)
-	     == scm_i_string_ref (suffix, j)))
-    {
-      --i;
-      --j;
-    }
-  if (j == -1)
-    end = i;
-  while (i >= 0 && !is_file_name_separator (scm_c_string_ref (filename, i)))
-    --i;
-  if (i == end)
-    {
-      if (len > 0 && is_file_name_separator (scm_c_string_ref (filename, 0)))
-        return scm_c_substring (filename, 0, 1);
-      else
-	return scm_dot_string;
-    }
-  else
-    return scm_c_substring (filename, i+1, end+1);
+    res = scm_from_utf8_string (c_last_component);
+  scm_dynwind_end ();
+
+  if (!SCM_UNBNDP (suffix) &&
+      scm_is_true (scm_string_suffix_p (suffix, filename,
+                                        SCM_UNDEFINED, SCM_UNDEFINED,
+                                        SCM_UNDEFINED, SCM_UNDEFINED)))
+    res = scm_c_substring
+      (res, 0, scm_c_string_length (res) - scm_c_string_length (suffix));
+
+  return res;
 }
 #undef FUNC_NAME
 
@@ -1614,22 +1628,40 @@ SCM_DEFINE (scm_canonicalize_path, "canonicalize-path", 1, 0, 0,
 SCM
 scm_i_relativize_path (SCM path, SCM in_path)
 {
-  char *str, *canon;
   SCM scanon;
   
-  str = scm_to_locale_string (path);
-  canon = canonicalize_file_name (str);
-  free (str);
+  {
+    char *str, *canon;
+
+    str = scm_to_locale_string (path);
+    canon = canonicalize_file_name (str);
+    free (str);
+
+    if (!canon)
+      return SCM_BOOL_F;
+
+    scanon = scm_take_locale_string (canon);
+  }
   
-  if (!canon)
-    return SCM_BOOL_F;
-
-  scanon = scm_take_locale_string (canon);
-
   for (; scm_is_pair (in_path); in_path = scm_cdr (in_path))
     {
       SCM dir = scm_car (in_path);
-      size_t len = scm_c_string_length (dir);
+      size_t len;
+
+      /* Try to canonicalize DIR, since we have canonicalized PATH.  */
+      {
+        char *str, *canon;
+
+        str = scm_to_locale_string (dir);
+        canon = canonicalize_file_name (str);
+        free (str);
+  
+        if (canon)
+          dir = scm_from_locale_string (canon);
+        free (canon);
+      }
+
+      len = scm_c_string_length (dir);
 
       /* When DIR is empty, it means "current working directory".  We
 	 could set DIR to (getcwd) in that case, but then the
