@@ -53,6 +53,8 @@ SCM_API scm_t_port_type *scm_file_port_type;
 
 
 SCM_API void scm_evict_ports (int fd);
+SCM_INTERNAL int scm_i_mode_to_open_flags (SCM mode, int *is_binary,
+                                           const char *FUNC_NAME);
 SCM_API SCM scm_open_file_with_encoding (SCM filename, SCM modes,
                                          SCM guess_encoding, SCM encoding);
 SCM_API SCM scm_open_file (SCM filename, SCM modes);
