@@ -37,6 +37,7 @@
 #include "libguile/alist.h"
 #include "libguile/arbiters.h"
 #include "libguile/async.h"
+#include "libguile/atomic.h"
 #include "libguile/backtrace.h"
 #include "libguile/bitvectors.h"
 #include "libguile/boolean.h"
@@ -56,6 +57,7 @@
 #include "libguile/eval.h"
 #include "libguile/evalext.h"
 #include "libguile/expand.h"
+#include "libguile/fdes-finalizers.h"
 #include "libguile/feature.h"
 #include "libguile/filesys.h"
 #include "libguile/finalizers.h"
@@ -397,7 +399,9 @@ scm_i_init_guile (void *base)
   scm_bootstrap_loader ();
   scm_bootstrap_programs ();
   scm_bootstrap_vm ();
+  scm_register_atomic ();
   scm_register_r6rs_ports ();
+  scm_register_fdes_finalizers ();
   scm_register_foreign ();
   scm_register_foreign_object ();
   scm_register_srfi_1 ();
