@@ -150,6 +150,8 @@
          (emit-cached-module-box asm (from-sp dst)
                                  (constant mod) (constant name)
                                  (constant public?) (constant bound?)))
+        (($ $primcall 'define! (sym))
+         (emit-define! asm (from-sp dst) (from-sp (slot sym))))
         (($ $primcall 'resolve (name bound?))
          (emit-resolve asm (from-sp dst) (constant bound?)
                        (from-sp (slot name))))
@@ -258,6 +260,17 @@
         (($ $primcall 'bv-f64-ref (bv idx val))
          (emit-bv-f64-ref asm (from-sp dst) (from-sp (slot bv))
                           (from-sp (slot idx))))
+        (($ $primcall 'make-atomic-box (init))
+         (emit-make-atomic-box asm (from-sp dst) (from-sp (slot init))))
+        (($ $primcall 'atomic-box-ref (box))
+         (emit-atomic-box-ref asm (from-sp dst) (from-sp (slot box))))
+        (($ $primcall 'atomic-box-swap! (box val))
+         (emit-atomic-box-swap! asm (from-sp dst) (from-sp (slot box))
+                                (from-sp (slot val))))
+        (($ $primcall 'atomic-box-compare-and-swap! (box expected desired))
+         (emit-atomic-box-compare-and-swap!
+          asm (from-sp dst) (from-sp (slot box))
+          (from-sp (slot expected)) (from-sp (slot desired))))
         (($ $primcall name args)
          ;; FIXME: Inline all the cases.
          (let ((inst (prim-instruction name)))
@@ -312,8 +325,6 @@
          (emit-set-car! asm (from-sp (slot pair)) (from-sp (slot value))))
         (($ $primcall 'set-cdr! (pair value))
          (emit-set-cdr! asm (from-sp (slot pair)) (from-sp (slot value))))
-        (($ $primcall 'define! (sym value))
-         (emit-define! asm (from-sp (slot sym)) (from-sp (slot value))))
         (($ $primcall 'push-fluid (fluid val))
          (emit-push-fluid asm (from-sp (slot fluid)) (from-sp (slot val))))
         (($ $primcall 'pop-fluid ())
@@ -351,7 +362,9 @@
          (emit-bv-f64-set! asm (from-sp (slot bv)) (from-sp (slot idx))
                            (from-sp (slot val))))
         (($ $primcall 'unwind ())
-         (emit-unwind asm))))
+         (emit-unwind asm))
+        (($ $primcall 'atomic-box-set! (box val))
+         (emit-atomic-box-set! asm (from-sp (slot box)) (from-sp (slot val))))))
 
     (define (compile-values label exp syms)
       (match exp
