@@ -418,7 +418,7 @@ is or might be a read or a write to the same location as A."
   ((resolve name bound?)           (&read-object &module)      &type-check)
   ((cached-toplevel-box scope name bound?)                     &type-check)
   ((cached-module-box mod name public? bound?)                 &type-check)
-  ((define! name val)              (&read-object &module) (&write-object &box)))
+  ((define! name)                  (&read-object &module)))
 
 ;; Numbers.
 (define-primitive-effects
@@ -475,6 +475,7 @@ is or might be a read or a write to the same location as A."
   ((lognot . _)                    &type-check)
   ((ulogand . _))
   ((ulogior . _))
+  ((ulogxor . _))
   ((ulogsub . _))
   ((ursh . _))
   ((ulsh . _))
@@ -487,13 +488,13 @@ is or might be a read or a write to the same location as A."
 
 ;; Characters.
 (define-primitive-effects
-  ((char<? . _)                    &type-check)
-  ((char<=? . _)                   &type-check)
-  ((char>=? . _)                   &type-check)
-  ((char>? . _)                    &type-check)
   ((integer->char _)               &type-check)
   ((char->integer _)               &type-check))
 
+;; Atomics are a memory and a compiler barrier; they cause all effects
+;; so no need to have a case for them here.  (Though, see
+;; https://jfbastien.github.io/no-sane-compiler/.)
+
 (define (primitive-effects constants name args)
   (let ((proc (hashq-ref *primitive-effects* name)))
     (if proc
