@@ -493,9 +493,12 @@
        (lambda (cps val)
          (with-cps cps
            (let$ k (adapt-arity k src 0))
+           (letv box)
+           (letk kset ($kargs ('box) (box)
+                        ($continue k src ($primcall 'box-set! (box val)))))
            ($ (with-cps-constants ((name name))
                 (build-term
-                  ($continue k src ($primcall 'define! (name val))))))))))
+                  ($continue kset src ($primcall 'define! (name))))))))))
 
     (($ <call> src proc args)
      (convert-args cps (cons proc args)
@@ -952,11 +955,12 @@ integer."
 (define *comp-module* (make-fluid))
 
 (define %warning-passes
-  `((unused-variable     . ,unused-variable-analysis)
-    (unused-toplevel     . ,unused-toplevel-analysis)
-    (unbound-variable    . ,unbound-variable-analysis)
-    (arity-mismatch      . ,arity-analysis)
-    (format              . ,format-analysis)))
+  `((unused-variable             . ,unused-variable-analysis)
+    (unused-toplevel             . ,unused-toplevel-analysis)
+    (unbound-variable            . ,unbound-variable-analysis)
+    (macro-use-before-definition . ,macro-use-before-definition-analysis)
+    (arity-mismatch              . ,arity-analysis)
+    (format                      . ,format-analysis)))
 
 (define (optimize-tree-il x e opts)
   (define warnings
