@@ -92,7 +92,6 @@
 (define (singly-valued-expression? exp)
   (match exp
     (($ <const>) #t)
-    (($ <lexical-ref>) #t)
     (($ <void>) #t)
     (($ <lexical-ref>) #t)
     (($ <primitive-ref>) #t)
@@ -511,7 +510,15 @@ top-level bindings from ENV and return the resulting expression."
         (lambda ()
           (call-with-values
               (lambda ()
-                (apply (module-ref the-scm-module name) args))
+                (case name
+                  ((eq? eqv?)
+                   ;; Constants will be deduplicated later, but eq?
+                   ;; folding can happen now.  Anticipate the
+                   ;; deduplication by using equal? instead of eq?.
+                   ;; Same for eqv?.
+                   (apply equal? args))
+                  (else
+                   (apply (module-ref the-scm-module name) args))))
             (lambda results
               (values #t results))))
         (lambda _
@@ -1014,10 +1021,6 @@ top-level bindings from ENV and return the resulting expression."
        ;; reconstruct the let-values, pevaling the consumer.
        (let ((producer (for-values producer)))
          (or (match consumer
-               (($ <lambda-case> src (req-name) #f #f #f () (req-sym) body #f)
-                (for-tail
-                 (make-let src (list req-name) (list req-sym) (list producer)
-                           body)))
                ((and ($ <lambda-case> src () #f rest #f () (rest-sym) body #f)
                      (? (lambda _ (singly-valued-expression? producer))))
                 (let ((tmp (gensym "tmp ")))
