@@ -171,7 +171,7 @@
     not
     pair? null? nil? list?
     symbol? variable? vector? struct? string? number? char?
-    bytevector? keyword? bitvector?
+    bytevector? keyword? bitvector? atomic-box?
     complex? real? rational? inf? nan? integer? exact? inexact? even? odd?
     char<? char<=? char>=? char>?
     integer->char char->integer number->string string->number
@@ -194,7 +194,7 @@
     pair? null? nil? list?
     symbol? variable? vector? struct? string? number? char?
     bytevector? keyword? bitvector?
-    procedure? thunk?
+    procedure? thunk? atomic-box?
     acons cons cons* list vector))
 
 ;; Primitives that don't always return one value.
@@ -549,6 +549,24 @@
                         (chained-comparison-expander prim-name)))
           '(< > <= >= =))
 
+(define (character-comparison-expander char< <)
+  (lambda (src . args)
+    (expand-primcall
+     (make-primcall src <
+                    (map (lambda (arg)
+                           (make-primcall src 'char->integer (list arg)))
+                         args)))))
+
+(for-each (match-lambda
+            ((char< . <)
+             (hashq-set! *primitive-expand-table* char<
+                         (character-comparison-expander char< <))))
+          '((char<? . <)
+            (char>? . >)
+            (char<=? . <=)
+            (char>=? . >=)
+            (char=? . =)))
+
 ;; Appropriate for use with either 'eqv?' or 'equal?'.
 (define (maybe-simplify-to-eq prim)
   (case-lambda
@@ -579,7 +597,12 @@
 (define (expand-chained-comparisons prim)
   (case-lambda
     ((src) (make-const src #t))
-    ((src a) (make-const src #t))
+    ((src a)
+     ;; (< x) -> (begin (< x 0) #t).  Residualizes side-effects from x
+     ;; and, for numeric comparisons, checks that x is a number.
+     (make-seq src
+               (make-primcall src prim (list a (make-const src 0)))
+               (make-const src #t)))
     ((src a b) #f)
     ((src a b . rest)
      (make-conditional src (make-primcall src prim (list a b))
