@@ -243,29 +243,29 @@ the matching bits, possibly with bitwise operations to extract it from BITS."
 (define %tc8-flag (+ %tc3-imm24 0))
 
 ;; Cell types.
-(define %tc3-struct 1)
-(define %tc7-symbol 5)
-(define %tc7-variable 7)
-(define %tc7-vector 13)
-(define %tc7-wvect 15)
-(define %tc7-string 21)
-(define %tc7-number 23)
-(define %tc7-hashtable 29)
-(define %tc7-pointer 31)
-(define %tc7-fluid 37)
-(define %tc7-stringbuf 39)
-(define %tc7-dynamic-state 45)
-(define %tc7-frame 47)
-(define %tc7-keyword 53)
-(define %tc7-program 69)
-(define %tc7-vm-continuation 71)
-(define %tc7-bytevector 77)
-(define %tc7-weak-set 85)
-(define %tc7-weak-table 87)
-(define %tc7-array 93)
-(define %tc7-bitvector 95)
-(define %tc7-port 125)
-(define %tc7-smob 127)
+(define %tc3-struct #x01)
+(define %tc7-symbol #x05)
+(define %tc7-variable #x07)
+(define %tc7-vector #x0d)
+(define %tc7-wvect #x0f)
+(define %tc7-string #x15)
+(define %tc7-number #x17)
+(define %tc7-hashtable #x1d)
+(define %tc7-pointer #x1f)
+(define %tc7-fluid #x25)
+(define %tc7-stringbuf #x27)
+(define %tc7-dynamic-state #x2d)
+(define %tc7-frame #x2f)
+(define %tc7-keyword #x35)
+(define %tc7-program #x45)
+(define %tc7-vm-continuation #x47)
+(define %tc7-bytevector #x4d)
+(define %tc7-weak-set #x55)
+(define %tc7-weak-table #x57)
+(define %tc7-array #x5d)
+(define %tc7-bitvector #x5f)
+(define %tc7-port #x7d)
+(define %tc7-smob #x77)
 
 (define %tc16-bignum (+ %tc7-number (* 1 256)))
 (define %tc16-real (+ %tc7-number (* 2 256)))
