@@ -108,7 +108,7 @@
     (define (parse-tail-word word type)
       (with-syntax ((word word))
         (case type
-          ((C32 I32 A32 B32)
+          ((C32 I32 A32 B32 AU32 BU32 AS32 BS32 AF32 BF32)
            #'(word))
           ((N32 R32 L32 LO32)
            #'((unpack-s32 word)))
@@ -193,8 +193,11 @@ address of that offset."
     (((or 'br
           'br-if-nargs-ne 'br-if-nargs-lt 'br-if-nargs-gt
           'br-if-true 'br-if-null 'br-if-nil 'br-if-pair 'br-if-struct
-          'br-if-char 'br-if-eq 'br-if-eqv 'br-if-equal
+          'br-if-char 'br-if-eq 'br-if-eqv
           'br-if-= 'br-if-< 'br-if-<= 'br-if-> 'br-if->=
+          'br-if-u64-= 'br-if-u64-< 'br-if-u64-<=
+          'br-if-u64-<-scm 'br-if-u64-<=-scm 'br-if-u64-=-scm
+          'br-if-u64->-scm 'br-if-u64->=-scm
           'br-if-logtest) _ ... target)
      (list "-> ~A" (vector-ref labels (- (+ offset target) start))))
     (('br-if-tc7 slot invert? tc7 target)
@@ -224,6 +227,10 @@ address of that offset."
      (list "~a slot~:p" nlocals))
     (('reset-frame nlocals)
      (list "~a slot~:p" nlocals))
+    (('return-values nlocals)
+     (if (zero? nlocals)
+         (list "all values")
+         (list "~a value~:p" (1- nlocals))))
     (('bind-rest dst)
      (list "~a slot~:p" (1+ dst)))
     (('tail-call nargs proc)
@@ -291,8 +298,11 @@ address of that offset."
                  ((br
                    br-if-nargs-ne br-if-nargs-lt br-if-nargs-gt
                    br-if-true br-if-null br-if-nil br-if-pair br-if-struct
-                   br-if-char br-if-tc7 br-if-eq br-if-eqv br-if-equal
-                   br-if-= br-if-< br-if-<= br-if-> br-if->= br-if-logtest)
+                   br-if-char br-if-tc7 br-if-eq br-if-eqv
+                   br-if-= br-if-< br-if-<= br-if-> br-if->= br-if-logtest
+                   br-if-u64-= br-if-u64-< br-if-u64-<=
+                   br-if-u64-<-scm br-if-u64-<=-scm br-if-u64-=-scm
+                   br-if-u64->-scm br-if-u64->=-scm)
                   (match arg
                     ((_ ... target)
                      (add-label! (+ offset target) "L"))))
@@ -510,7 +520,7 @@ address of that offset."
   (define non-fallthrough-set
     (static-opcode-set halt
                        tail-call tail-call-label tail-call/shuffle
-                       return return-values
+                       return-values
                        subr-call foreign-call continuation-call
                        tail-apply
                        br))
