@@ -1,6 +1,6 @@
 ;;; Guile VM frame functions
 
-;;; Copyright (C) 2001, 2005, 2009, 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+;;; Copyright (C) 2001, 2005, 2009-2016 Free Software Foundation, Inc.
 ;;;
 ;;; This library is free software; you can redistribute it and/or
 ;;; modify it under the terms of the GNU Lesser General Public
@@ -31,20 +31,29 @@
   #:export (binding-index
             binding-name
             binding-slot
+            binding-representation
 
             frame-bindings
             frame-lookup-binding
-            frame-binding-ref frame-binding-set!
+            binding-ref binding-set!
+
+            frame-instruction-pointer-or-primitive-procedure-name
             frame-call-representation
             frame-environment
             frame-object-binding frame-object-name))
 
+(eval-when (expand compile load eval)
+  (load-extension (string-append "libguile-" (effective-version))
+                  "scm_init_frames_builtins"))
+
 (define-record-type <binding>
-  (make-binding idx name slot)
+  (make-binding frame idx name slot representation)
   binding?
+  (frame binding-frame)
   (idx binding-index)
   (name binding-name)
-  (slot binding-slot))
+  (slot binding-slot)
+  (representation binding-representation))
 
 (define (parse-code code)
   (let ((len (bytevector-length code)))
@@ -134,7 +143,7 @@
     (let lp ((var 0) (pos 0) (pc-offset 0))
       (when (< var (vector-length defs))
         (match (vector-ref defs var)
-          (#(name offset slot)
+          (#(name offset slot representation)
            (when (< offset pc-offset)
              (error "mismatch between def offsets and parsed code"))
            (cond
@@ -147,7 +156,7 @@
 
 (define (compute-defs-by-slot defs)
   (let* ((nslots (match defs
-                   (#(#(_ _ slot) ...) (1+ (apply max slot)))))
+                   (#(#(_ _ slot _) ...) (1+ (apply max slot)))))
          (by-slot (make-vector nslots #f)))
     (let lp ((n 0))
       (when (< n nslots)
@@ -156,7 +165,7 @@
     (let lp ((n 0))
       (when (< n (vector-length defs))
         (match (vector-ref defs n)
-          (#(_ _ slot)
+          (#(_ _ slot _)
            (bitvector-set! (vector-ref by-slot slot) n #t)
            (lp (1+ n))))))
     by-slot))
@@ -179,7 +188,7 @@
     (let lp ((var 0) (pos 0) (pc-offset 0))
       (when (< var (vector-length defs))
         (match (vector-ref defs var)
-          (#(name offset slot)
+          (#(name offset slot representation)
            (when (< offset pc-offset)
              (error "mismatch between def offsets and parsed code"))
            (cond
@@ -199,7 +208,7 @@
         (lp (1+ n) (+ pos (vector-ref parsed n)))))
     killv))
 
-(define (available-bindings arity ip top-frame?)
+(define (available-bindings frame arity ip top-frame?)
   (let* ((defs (list->vector (arity-definitions arity)))
          (code (arity-code arity))
          (parsed (parse-code code))
@@ -274,10 +283,8 @@
               (let ((n (bit-position #t live n)))
                 (if n
                     (match (vector-ref defs n)
-                      (#(name def-offset slot)
-                       ;; Binding 0 is the closure, and is not present
-                       ;; in arity-definitions.
-                       (cons (make-binding (1+ n) name slot)
+                      (#(name def-offset slot representation)
+                       (cons (make-binding frame n name slot representation)
                              (lp (1+ n)))))
                     '()))))
           (lp (1+ n) (- offset (vector-ref parsed n)))))))
@@ -287,7 +294,7 @@
     (cond
      ((find-program-arity ip)
       => (lambda (arity)
-           (available-bindings arity ip top-frame?)))
+           (available-bindings frame arity ip top-frame?)))
      (else '()))))
 
 (define (frame-lookup-binding frame var)
@@ -299,19 +306,40 @@
           (else
            (lp (cdr bindings))))))
 
-(define (frame-binding-set! frame var val)
-  (frame-local-set! frame
-                    (binding-slot
-                     (or (frame-lookup-binding frame var)
-                         (error "variable not bound in frame" var frame)))
-                    val))
+(define (binding-ref binding)
+  (frame-local-ref (or (binding-frame binding)
+                       (error "binding has no frame" binding))
+                   (binding-slot binding)
+                   (binding-representation binding)))
 
-(define (frame-binding-ref frame var)
-  (frame-local-ref frame
-                   (binding-slot
-                    (or (frame-lookup-binding frame var)
-                        (error "variable not bound in frame" var frame)))))
+(define (binding-set! binding val)
+  (frame-local-set! (or (binding-frame binding)
+                        (error "binding has no frame" binding))
+                    (binding-slot binding)
+                    val
+                    (binding-representation binding)))
 
+(define* (frame-procedure-name frame #:key
+                               (info (find-program-debug-info
+                                      (frame-instruction-pointer frame))))
+  (cond
+   (info => program-debug-info-name)
+   ;; We can only try to get the name from the closure if we know that
+   ;; slot 0 corresponds to the frame's procedure.  This isn't possible
+   ;; to know in general.  If the frame has already begun executing and
+   ;; the closure binding is dead, it could have been replaced with any
+   ;; other random value, or an unboxed value.  Even if we're catching
+   ;; the frame at its application, before it has started running, if
+   ;; the callee is well-known and has only one free variable, closure
+   ;; optimization could have chosen to represent its closure as that
+   ;; free variable, and that free variable might be some other program,
+   ;; or even an unboxed value.  It would be an error to try to get the
+   ;; procedure name of some procedure that doesn't correspond to the
+   ;; one being applied.  (Free variables are currently always boxed but
+   ;; that could change in the future.)
+   ((primitive-code? (frame-instruction-pointer frame))
+    (procedure-name (frame-local-ref frame 0 'scm)))
+   (else #f)))
 
 ;; This function is always called to get some sort of representation of the
 ;; frame to present to the user, so let's do the logical thing and dispatch to
@@ -319,6 +347,16 @@
 (define (frame-arguments frame)
   (cdr (frame-call-representation frame)))
 
+;; Usually the IP is sufficient to identify the procedure being called.
+;; However all primitive applications of the same arity share the same
+;; code.  Perhaps we should change that in the future, but for now we
+;; export this function to avoid having to export frame-local-ref.
+;;
+(define (frame-instruction-pointer-or-primitive-procedure-name frame)
+  (let ((ip (frame-instruction-pointer frame)))
+    (if (primitive-code? ip)
+        (procedure-name (frame-local-ref frame 0 'scm))
+        ip)))
 
 
 ;;;
@@ -343,20 +381,33 @@
 (define* (frame-call-representation frame #:key top-frame?)
   (let* ((ip (frame-instruction-pointer frame))
          (info (find-program-debug-info ip))
-         (nlocals (frame-num-locals frame))
-         (closure (frame-procedure frame)))
+         (nlocals (frame-num-locals frame)))
     (define (find-slot i bindings)
       (match bindings
-        (#f (and (< i nlocals) i))
         (() #f)
-        ((($ <binding> idx name slot) . bindings)
+        (((and binding ($ <binding> frame idx name slot)) . bindings)
          (if (< idx i)
              (find-slot i bindings)
-             (and (= idx i) slot)))))
+             (and (= idx i) binding)))))
     (define (local-ref i bindings)
       (cond
+       ((not bindings)
+        ;; This case is only hit for primitives and application
+        ;; arguments.
+        (frame-local-ref frame i 'scm))
        ((find-slot i bindings)
-        => (lambda (slot) (frame-local-ref frame slot)))
+        => (lambda (binding)
+             (let ((val (frame-local-ref frame (binding-slot binding)
+                                         (binding-representation binding))))
+               ;; It could be that there's a value that isn't clobbered
+               ;; by a call but that isn't live after a call either.  In
+               ;; that case, if GC runs during the call, the value will
+               ;; be collected, and on the stack it will be replaced
+               ;; with the unspecified value.  Assume that clobbering
+               ;; values is more likely than passing the unspecified
+               ;; value as an argument, and replace unspecified with _,
+               ;; as if the binding were not available.
+               (if (unspecified? val) '_ val))))
        (else
         '_)))
     (define (application-arguments)
@@ -384,22 +435,21 @@
        (else
         '())))
     (cons
-     (or (and=> info program-debug-info-name)
-         (and (procedure? closure) (procedure-name closure))
-         closure)
+     (or (frame-procedure-name frame #:info info) '_)
      (cond
       ((find-program-arity ip)
        => (lambda (arity)
             (if (and top-frame? (eqv? ip (arity-low-pc arity)))
                 (application-arguments)
-                (reconstruct-arguments (available-bindings arity ip top-frame?)
-                                       (arity-nreq arity)
-                                       (arity-nopt arity)
-                                       (arity-keyword-args arity)
-                                       (arity-has-rest? arity)
-                                       1))))
-      ((and (primitive? closure)
-            (program-arguments-alist closure ip))
+                (reconstruct-arguments
+                 (available-bindings frame arity ip top-frame?)
+                 (arity-nreq arity)
+                 (arity-nopt arity)
+                 (arity-keyword-args arity)
+                 (arity-has-rest? arity)
+                 1))))
+      ((and (primitive-code? ip)
+            (program-arguments-alist (frame-local-ref frame 0 'scm) ip))
        => (lambda (args)
             (match args
               ((('required . req)
@@ -419,12 +469,12 @@
 
 (define (frame-environment frame)
   (map (lambda (binding)
-	 (cons (binding-name binding) (frame-binding-ref frame binding)))
+	 (cons (binding-name binding) (binding-ref binding)))
        (frame-bindings frame)))
 
 (define (frame-object-binding frame obj)
   (do ((bs (frame-bindings frame) (cdr bs)))
-      ((or (null? bs) (eq? obj (frame-binding-ref frame (car bs))))
+      ((or (null? bs) (eq? obj (binding-ref (car bs))))
        (and (pair? bs) (car bs)))))
 
 (define (frame-object-name frame obj)
